@@ -6,8 +6,8 @@ use crate::rulebook as rb;
 use crate::runtime as rt;
 use std::io::Write;
 
-pub fn compile_code_and_save(code: &str, file_name: &str) -> std::io::Result<()> {
-  let as_clang = compile_code(code);
+pub fn compile_code_and_save(code: &str, file_name: &str, parallel: bool) -> std::io::Result<()> {
+  let as_clang = compile_code(code, parallel);
   let mut file = std::fs::OpenOptions::new()
     .read(true)
     .write(true)
@@ -18,19 +18,19 @@ pub fn compile_code_and_save(code: &str, file_name: &str) -> std::io::Result<()>
   Ok(())
 }
 
-pub fn compile_code(code: &str) -> String {
+pub fn compile_code(code: &str, parallel: bool) -> String {
   let file = lang::read_file(code);
   let book = rb::gen_rulebook(&file);
   let _funs = bd::build_runtime_functions(&book);
-  compile_book(&book)
+  compile_book(&book, parallel)
 }
 
 pub fn compile_name(name: &str) -> String {
   str::replace(&format!("_{}_", name.to_uppercase()), ".", "$")
 }
 
-pub fn compile_book(comp: &rb::RuleBook) -> String {
-  let mut dups  = 0;
+pub fn compile_book(comp: &rb::RuleBook, parallel: bool) -> String {
+  let mut dups = 0;
   let mut c_ids = String::new();
   let mut inits = String::new();
   let mut codes = String::new();
@@ -57,10 +57,15 @@ pub fn compile_book(comp: &rb::RuleBook) -> String {
     line(&mut codes, 6, "};");
   }
 
-  c_runtime_template(&c_ids, &inits, &codes, &id2nm, comp.id_to_name.len() as u64)
+  c_runtime_template(&c_ids, &inits, &codes, &id2nm, comp.id_to_name.len() as u64, parallel)
 }
 
-pub fn compile_func(comp: &rb::RuleBook, rules: &[lang::Rule], tab: u64, dups: &mut u64) -> (String, String) {
+pub fn compile_func(
+  comp: &rb::RuleBook,
+  rules: &[lang::Rule],
+  tab: u64,
+  dups: &mut u64,
+) -> (String, String) {
   let dynfun = bd::build_dynfun(comp, rules);
 
   let mut init = String::new();
@@ -82,11 +87,15 @@ pub fn compile_func(comp: &rb::RuleBook, rules: &[lang::Rule], tab: u64, dups: &
     line(&mut init, tab + 1, "init = 0;");
   } else {
     line(&mut init, tab + 1, "stk_push(&stack, host);");
-    for i in 0 .. stricts.len() {
+    for (i, strict) in stricts.iter().enumerate() {
       if i < stricts.len() - 1 {
-        line(&mut init, tab + 1, &format!("stk_push(&stack, get_loc(term, {}) | 0x80000000);", stricts[i]));
+        line(
+          &mut init,
+          tab + 1,
+          &format!("stk_push(&stack, get_loc(term, {}) | 0x80000000);", strict),
+        );
       } else {
-        line(&mut init, tab + 1, &format!("host = get_loc(term, {});", stricts[i]));
+        line(&mut init, tab + 1, &format!("host = get_loc(term, {});", strict));
       }
     }
   }
@@ -97,8 +106,12 @@ pub fn compile_func(comp: &rb::RuleBook, rules: &[lang::Rule], tab: u64, dups: &
   for (i, is_redex) in dynfun.redex.iter().enumerate() {
     if *is_redex {
       line(&mut code, tab + 0, &format!("if (get_tag(ask_arg(mem,term,{})) == PAR) {{", i));
-      line(&mut code, tab + 1, &format!("cal_par(mem, host, term, ask_arg(mem, term, {}), {});", i, i));
-      line(&mut code, tab + 1, &format!("continue;"));
+      line(
+        &mut code,
+        tab + 1,
+        &format!("cal_par(mem, host, term, ask_arg(mem, term, {}), {});", i, i),
+      );
+      line(&mut code, tab + 1, "continue;");
       line(&mut code, tab + 0, "}");
     }
   }
@@ -292,7 +305,11 @@ pub fn compile_func_rule_term(
         line(code, tab + 0, &format!("u64 {};", retx));
         // Optimization: do inline operation, avoiding Op2 allocation, when operands are already number
         if INLINE_NUMBERS {
-          line(code, tab + 0, &format!("if (get_tag({}) == U32 && get_tag({}) == U32) {{", val0, val1));
+          line(
+            code,
+            tab + 0,
+            &format!("if (get_tag({}) == U32 && get_tag({}) == U32) {{", val0, val1),
+          );
           let a = format!("get_val({})", val0);
           let b = format!("get_val({})", val1);
           match *oper {
@@ -444,8 +461,10 @@ pub fn c_runtime_template(
   codes: &str,
   id2nm: &str,
   names_count: u64,
+  parallel: bool,
 ) -> String {
   const C_RUNTIME_TEMPLATE: &str = include_str!("runtime.c");
+  const C_PARALLEL_FLAG_CONTENT: &str = "/* GENERATED_PARALLEL_FLAG_CONTENT */";
   const C_CONSTRUCTOR_IDS_CONTENT: &str = "/* GENERATED_CONSTRUCTOR_IDS_CONTENT */";
   const C_REWRITE_RULES_STEP_0_CONTENT: &str = "/* GENERATED_REWRITE_RULES_STEP_0_CONTENT */";
   const C_REWRITE_RULES_STEP_1_CONTENT: &str = "/* GENERATED_REWRITE_RULES_STEP_1_CONTENT */";
@@ -460,6 +479,11 @@ pub fn c_runtime_template(
     "The runtime C file is missing the constructor ids section token: {}",
     C_CONSTRUCTOR_IDS_CONTENT
   );
+  debug_assert!(
+    C_RUNTIME_TEMPLATE.contains(C_PARALLEL_FLAG_CONTENT),
+    "The runtime C file is missing parallel flag token: {}",
+    C_PARALLEL_FLAG_CONTENT
+  );
   debug_assert!(
     C_RUNTIME_TEMPLATE.contains(C_REWRITE_RULES_STEP_0_CONTENT),
     "The runtime C file is missing the rewrite rules step 0 section token: {}",
@@ -489,6 +513,7 @@ pub fn c_runtime_template(
   // Instantiate the template with the given sections' content
 
   C_RUNTIME_TEMPLATE
+    .replace(C_PARALLEL_FLAG_CONTENT, if parallel { "#define PARALLEL" } else { "" })
     .replace(C_NUM_THREADS_CONTENT, &num_cpus::get().to_string())
     .replace(C_CONSTRUCTOR_IDS_CONTENT, c_ids)
     .replace(C_REWRITE_RULES_STEP_0_CONTENT, inits)
