@@ -30,14 +30,20 @@ fn run_cli() -> std::io::Result<()> {
 
   let cmd = &args[1];
 
+  if (cmd == "d" || cmd == "debug") && args.len() >= 3 {
+    let file = &hvm(&args[2]);
+    return run_code(&load_file_code(file), true);
+  }
+
   if (cmd == "r" || cmd == "run") && args.len() >= 3 {
     let file = &hvm(&args[2]);
-    return run_code(&load_file_code(file));
+    return run_code(&load_file_code(file), false);
   }
 
   if (cmd == "c" || cmd == "compile") && args.len() >= 3 {
     let file = &hvm(&args[2]);
-    return compile_code(&load_file_code(file), file);
+    let parallel = !(args.len() >= 4 && args[3] == "--single-thread");
+    return compile_code(&load_file_code(file), file, parallel);
   }
 
   println!("Invalid arguments: {:?}.", args);
@@ -50,13 +56,17 @@ fn show_help() {
   println!();
   println!("To run a file, interpreted:");
   println!();
-  println!("  hvm run file.hvm");
+  println!("  hvm r file.hvm");
+  println!();
+  println!("To run a file in debug mode:");
+  println!();
+  println!("  hvm d file.hvm");
   println!();
   println!("To compile a file to C:");
   println!();
-  println!("  hvm c file.hvm");
+  println!("  hvm c file.hvm [--single-thread]");
   println!();
-  println!("More info: https://github.com/kindelia/hvm");
+  println!("This is a PROTOTYPE. Report bugs on https://github.com/Kindelia/HVM/issues!");
   println!();
 }
 
@@ -67,9 +77,9 @@ fn make_call() -> language::Term {
   language::Term::Ctr { name, args }
 }
 
-fn run_code(code: &str) -> std::io::Result<()> {
+fn run_code(code: &str, debug: bool) -> std::io::Result<()> {
   println!("Reducing.");
-  let (norm, cost, size, time) = builder::eval_code(&make_call(), code);
+  let (norm, cost, size, time) = builder::eval_code(&make_call(), code, debug);
   println!("Rewrites: {} ({:.2} MR/s)", cost, (cost as f64) / (time as f64) / 1000.0);
   println!("Mem.Size: {}", size);
   println!();
@@ -77,12 +87,12 @@ fn run_code(code: &str) -> std::io::Result<()> {
   Ok(())
 }
 
-fn compile_code(code: &str, name: &str) -> std::io::Result<()> {
+fn compile_code(code: &str, name: &str, parallel: bool) -> std::io::Result<()> {
   if !name.ends_with(".hvm") {
     panic!("Input file must end with .hvm.");
   }
   let name = format!("{}.c", &name[0..name.len() - 4]);
-  compiler::compile_code_and_save(code, &name)?;
+  compiler::compile_code_and_save(code, &name, parallel)?;
   println!("Compiled to '{}'.", name);
   Ok(())
 }
@@ -140,14 +150,14 @@ fn run_example() -> std::io::Result<()> {
   ";
 
   // Compiles to C and saves as 'main.c'
-  compiler::compile_code_and_save(code, "main.c")?;
+  compiler::compile_code_and_save(code, "main.c", true)?;
   println!("Compiled to 'main.c'.");
 
   // Evaluates with interpreter
 
   println!("Reducing with interpreter.");
   let call = language::Term::Ctr { name: "Main".to_string(), args: Vec::new() };
-  let (norm, cost, size, time) = builder::eval_code(&call, code);
+  let (norm, cost, size, time) = builder::eval_code(&call, code, false);
   println!("Rewrites: {} ({:.2} MR/s)", cost, (cost as f64) / (time as f64) / 1000.0);
   println!("Mem.Size: {}", size);
   println!();
