@@ -54,9 +54,6 @@ pub const GTE: u64 = 0xD;
 pub const GTN: u64 = 0xE;
 pub const NEQ: u64 = 0xF;
 
-const _MAIN: u64 = 0;
-const _SLOW: u64 = 1;
-
 // Types
 // -----
 
@@ -305,6 +302,7 @@ pub fn reduce(
   funcs: &[Option<Function>],
   root: u64,
   _opt_id_to_name: Option<&HashMap<u64, String>>,
+  debug: bool,
 ) -> Lnk {
   let mut stack: Vec<u64> = Vec::new();
 
@@ -313,12 +311,11 @@ pub fn reduce(
 
   loop {
     let term = ask_lnk(mem, host);
-    //println!("reduce {}", show_lnk(term));
-    //println!("------ {}", show_term(mem, ask_lnk(mem, root), _opt_id_to_name));
-    //for i in 0 .. 256 {
-      //println!("- {:x} {}", i, show_lnk(mem.node[i]));
-    //}
-    //println!("memory {}", show_mem(mem));
+
+    if debug {
+      println!("------------------------");
+      println!("{}", show_term(mem, ask_lnk(mem, root), _opt_id_to_name, term));
+    }
 
     if init == 1 {
       match get_tag(term) {
@@ -335,8 +332,8 @@ pub fn reduce(
         }
         OP2 => {
           stack.push(host);
-          stack.push(get_loc(term, 0) | 0x80000000);
-          host = get_loc(term, 1);
+          stack.push(get_loc(term, 1) | 0x80000000);
+          host = get_loc(term, 0);
           continue;
         }
         CAL => {
@@ -349,11 +346,11 @@ pub fn reduce(
                 init = 0;
               } else {
                 stack.push(host);
-                for i in 0 .. f.stricts.len() {
+                for (i, strict) in f.stricts.iter().enumerate() {
                   if i < f.stricts.len() - 1 {
-                    stack.push(get_loc(term, f.stricts[i]) | 0x80000000);
+                    stack.push(get_loc(term, *strict) | 0x80000000);
                   } else {
-                    host = get_loc(term, f.stricts[i]);
+                    host = get_loc(term, *strict);
                   }
                 }
               }
@@ -397,6 +394,13 @@ pub fn reduce(
         }
         DP0 | DP1 => {
           let arg0 = ask_arg(mem, term, 2);
+          //let argK = ask_arg(mem, term, if get_tag(term) == DP0 { 1 } else { 0 });
+          //if get_tag(argK) == ERA {
+            //let done = arg0;
+            //link(mem, host, done);
+            //init = 1;
+            //continue;
+          //}
           if get_tag(arg0) == LAM {
             //println!("dup-lam");
             inc_cost(mem);
@@ -425,7 +429,8 @@ pub fn reduce(
               inc_cost(mem);
               subst(mem, ask_arg(mem, term, 0), ask_arg(mem, arg0, 0));
               subst(mem, ask_arg(mem, term, 1), ask_arg(mem, arg0, 1));
-              let _done = link(mem, host, ask_arg(mem, arg0, if get_tag(term) == DP0 { 0 } else { 1 }));
+              let _done =
+                link(mem, host, ask_arg(mem, arg0, if get_tag(term) == DP0 { 0 } else { 1 }));
               clear(mem, get_loc(term, 0), 3);
               clear(mem, get_loc(arg0, 0), 2);
               init = 1;
@@ -469,7 +474,7 @@ pub fn reduce(
             } else {
               let ctr0 = get_loc(arg0, 0);
               let ctr1 = alloc(mem, arit);
-              for i in 0 .. arit - 1 {
+              for i in 0..arit - 1 {
                 let leti = alloc(mem, 3);
                 link(mem, leti + 2, ask_arg(mem, arg0, i));
                 link(mem, ctr0 + i, Dp0(get_ext(term), leti));
@@ -629,12 +634,13 @@ pub fn normal_go(
   host: u64,
   seen: &mut [u64],
   opt_id_to_name: Option<&HashMap<u64, String>>,
+  debug: bool,
 ) -> Lnk {
   let term = ask_lnk(mem, host);
   if get_bit(seen, host) {
     term
   } else {
-    let term = reduce(mem, funcs, host, opt_id_to_name);
+    let term = reduce(mem, funcs, host, opt_id_to_name, debug);
     set_bit(seen, host);
     let mut rec_locs = Vec::with_capacity(16);
     match get_tag(term) {
@@ -664,7 +670,7 @@ pub fn normal_go(
       _ => {}
     }
     for loc in rec_locs {
-      let lnk: Lnk = normal_go(mem, funcs, loc, seen, opt_id_to_name);
+      let lnk: Lnk = normal_go(mem, funcs, loc, seen, opt_id_to_name, debug);
       link(mem, loc, lnk);
     }
     term
@@ -676,9 +682,10 @@ pub fn normal(
   host: u64,
   funcs: &[Option<Function>],
   opt_id_to_name: Option<&HashMap<u64, String>>,
+  debug: bool,
 ) -> Lnk {
   let mut seen = vec![0; 4194304];
-  normal_go(mem, funcs, host, &mut seen, opt_id_to_name)
+  normal_go(mem, funcs, host, &mut seen, opt_id_to_name, debug)
 }
 
 // Debug
@@ -729,7 +736,7 @@ pub fn show_mem(worker: &Worker) -> String {
   s
 }
 
-pub fn show_term(mem: &Worker, term: Lnk, opt_id_to_name: Option<&HashMap<u64, String>>) -> String {
+pub fn show_term(mem: &Worker, term: Lnk, opt_id_to_name: Option<&HashMap<u64, String>>, focus: u64) -> String {
   let mut lets: HashMap<u64, u64> = HashMap::new();
   let mut kinds: HashMap<u64, u64> = HashMap::new();
   let mut names: HashMap<u64, String> = HashMap::new();
@@ -792,36 +799,37 @@ pub fn show_term(mem: &Worker, term: Lnk, opt_id_to_name: Option<&HashMap<u64, S
     term: Lnk,
     names: &HashMap<u64, String>,
     opt_id_to_name: Option<&HashMap<u64, String>>,
+    focus: u64,
   ) -> String {
-    match get_tag(term) {
+    let done = match get_tag(term) {
       DP0 => {
-        return format!("a{}", names.get(&get_loc(term, 0)).unwrap_or(&String::from("?")));
+        format!("a{}", names.get(&get_loc(term, 0)).unwrap_or(&String::from("?")))
       }
       DP1 => {
-        return format!("b{}", names.get(&get_loc(term, 0)).unwrap_or(&String::from("?")));
+        format!("b{}", names.get(&get_loc(term, 0)).unwrap_or(&String::from("?")))
       }
       VAR => {
-        return format!("x{}", names.get(&get_loc(term, 0)).unwrap_or(&String::from("?")));
+        format!("x{}", names.get(&get_loc(term, 0)).unwrap_or(&String::from("?")))
       }
       LAM => {
         let name = format!("x{}", names.get(&get_loc(term, 0)).unwrap_or(&String::from("?")));
-        return format!("λ{} {}", name, go(mem, ask_arg(mem, term, 1), names, opt_id_to_name));
+        format!("λ{} {}", name, go(mem, ask_arg(mem, term, 1), names, opt_id_to_name, focus))
       }
       APP => {
-        let func = go(mem, ask_arg(mem, term, 0), names, opt_id_to_name);
-        let argm = go(mem, ask_arg(mem, term, 1), names, opt_id_to_name);
-        return format!("({} {})", func, argm);
+        let func = go(mem, ask_arg(mem, term, 0), names, opt_id_to_name, focus);
+        let argm = go(mem, ask_arg(mem, term, 1), names, opt_id_to_name, focus);
+        format!("({} {})", func, argm)
       }
       PAR => {
-        let kind = get_ext(term);
-        let func = go(mem, ask_arg(mem, term, 0), names, opt_id_to_name);
-        let argm = go(mem, ask_arg(mem, term, 1), names, opt_id_to_name);
-        return format!("&{}<{} {}>", kind, func, argm);
+        //let kind = get_ext(term);
+        let func = go(mem, ask_arg(mem, term, 0), names, opt_id_to_name, focus);
+        let argm = go(mem, ask_arg(mem, term, 1), names, opt_id_to_name, focus);
+        format!("{{{} {}}}", func, argm)
       }
       OP2 => {
         let oper = get_ext(term);
-        let val0 = go(mem, ask_arg(mem, term, 0), names, opt_id_to_name);
-        let val1 = go(mem, ask_arg(mem, term, 1), names, opt_id_to_name);
+        let val0 = go(mem, ask_arg(mem, term, 0), names, opt_id_to_name, focus);
+        let val1 = go(mem, ask_arg(mem, term, 1), names, opt_id_to_name, focus);
         let symb = match oper {
           0x00 => "+",
           0x01 => "-",
@@ -841,16 +849,16 @@ pub fn show_term(mem: &Worker, term: Lnk, opt_id_to_name: Option<&HashMap<u64, S
           0x15 => "!=",
           _ => "?",
         };
-        return format!("({} {} {})", symb, val0, val1);
+        format!("({} {} {})", symb, val0, val1)
       }
       U32 => {
-        return format!("{}", get_val(term));
+        format!("{}", get_val(term))
       }
       CTR | CAL => {
         let func = get_ext(term);
         let arit = get_ari(term);
         let args: Vec<String> =
-          (0..arit).map(|i| go(mem, ask_arg(mem, term, i), names, opt_id_to_name)).collect();
+          (0..arit).map(|i| go(mem, ask_arg(mem, term, i), names, opt_id_to_name, focus)).collect();
         let name = if let Some(id_to_name) = opt_id_to_name {
           id_to_name.get(&func).unwrap_or(&String::from("?")).clone()
         } else {
@@ -860,28 +868,31 @@ pub fn show_term(mem: &Worker, term: Lnk, opt_id_to_name: Option<&HashMap<u64, S
             func
           )
         };
-        return format!("({}{})", name, args.iter().map(|x| format!(" {}", x)).collect::<String>());
+        format!("({}{})", name, args.iter().map(|x| format!(" {}", x)).collect::<String>())
       }
       _ => String::from("?"),
-    }
+    };
+    if term == focus {
+      return format!("${}", done);
+    } else {
+      return done;
+    };
   }
   find_lets(mem, term, &mut lets, &mut kinds, &mut names, &mut count);
-  let mut text = go(mem, term, &names, opt_id_to_name);
-  for (key, pos) in lets {
+  let mut text = go(mem, term, &names, opt_id_to_name, focus);
+  for (_key, pos) in lets {
     // todo: reverse
     let what = String::from("?");
-    let kind = kinds.get(&key).unwrap_or(&0);
+    //let kind = kinds.get(&key).unwrap_or(&0);
     let name = names.get(&pos).unwrap_or(&what);
-    let nam0 =
-      if ask_lnk(mem, pos + 0) == Era() { String::from("*") } else { format!("a{}", name) };
-    let nam1 =
-      if ask_lnk(mem, pos + 1) == Era() { String::from("*") } else { format!("b{}", name) };
+    let nam0 = if ask_lnk(mem, pos + 0) == Era() { String::from("*") } else { format!("a{}", name) };
+    let nam1 = if ask_lnk(mem, pos + 1) == Era() { String::from("*") } else { format!("b{}", name) };
     text.push_str(&format!(
-      " !{}<{} {}> = {};",
-      kind,
+      "\ndup {} {} = {};",
+      //kind,
       nam0,
       nam1,
-      go(mem, ask_lnk(mem, pos + 2), &names, opt_id_to_name)
+      go(mem, ask_lnk(mem, pos + 2), &names, opt_id_to_name, focus)
     ));
   }
   text
